/**
 * Copyright (C) 2003-2019, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 *
 * @file fs_signature.h
 * @brief Header file for PDF siganture related definitions and classes.
 */

#ifndef FS_SIGNATURE_H_
#define FS_SIGNATURE_H_

#include "common/fs_common.h"
#include "common/fs_image.h"
#include "pdf/interform/fs_pdfform.h"

/**
 * @brief Foxit namespace.
 *
 * @details All the definitions and classes of Foxit PDF SDK are defined within this namespace.
 */
namespace foxit {
/**
 * @brief PDF namespace.
 */
namespace pdf {
/**
 * A digital signature (PDF 1.3) can be used to authenticate the identity of a user and the document's contents.
 * It stores information about the signer and the state of the document when it was signed. The signature is
 * contained in a signature field, as a type of form field, so class @link Signature @endlink is derived from
 * class @link interform::Field @endlink.
 * A signature object can be retrieved/added by following functions:
 * <ul>
 * <li>To be counted and retrieved as a signature object directly from PDF document, please use functions
 *     @link pdf::PDFDoc::GetSignatureCount @endlink and @link PDFDoc::GetSignature @endlink. </li>
 * <li>To be counted and retrieved as a signature field, please use functions
 *     @link interform::Form::GetFieldCount @endlink and @link interform::Form::GetField @endlink when field type is
 *     @link interform::Field::e_TypeSignature @endlink.</li>
 * <li>To add a new signature, please use function @link PDFPage::AddSignature @endlink.</li>
 * <li>To remove a signature, please use function @link PDFDoc::RemoveSignature @endlink.</li>
 * </ul>
 * In a signature, filter and sub filter keys are used to specify the filter and sub filter of
 * signature callback object which will be used to sign and verify the signature. Foxit PDF SDK has default
 * signature callbacks for following filter and sub filter:
 * <ul>
 * <li>filter: "Adobe.PPKLite" and sub filter: "adbe.pkcs7.detached"</li>
 * <li>filter: "Adobe.PPKLite" and sub filter: "adbe.pkcs7.sha1"</li>
 * </ul>
 * If user wants to use customized signature callback, please refer to function
 * @link common::Library::RegisterSignatureCallback @endlink.<br>
 * This class offers functions to get/set signature information/properties, sign or verify a signature, and so on.
 * For example:
 * <ul>
 * <li>To sign an unsigned signature, please use functions @link Signature::StartSign @endlink.
 *     When signing a signature successfully, user is strongly recommended to close current document and
 *     then open the signed PDF document and then do following operation.</li>
 * <li>To verify a signed signature, please use functions @link Signature::StartVerify @endlink.</li>
 * <li>To retrieve signature dictionary directly, please use function @link Signature::GetSignatureDict @endlink.</li>
 * </ul>
 * Before signing an unsigned signature, user can call following functions to set information for signing and
 * signed appearance:<br>
 * <ul>
 * <li>For using default Foxit appearance template, related information can be set by
 *     @link Signature::SetAppearanceFlags @endlink, @link Signature::SetSignTime @endlink, @link Signature::SetKeyValue @endlink,
 *     @link Signature::SetBitmap @endlink, @link Signature::SetImage @endlink.</li>
 * <li>For using customized appearance, please set appearance stream content by function
 *     @link Signature::SetAppearanceContent @endlink.</li>
 * </ul>
 * If an unsigned signature has been set some information for signing, but the document is saved directly or
 * closed without signing the signature, these data (including filter and sub filter) will be lost
 * in the saved document or in the closed document. When the document is opened again, the unsigned signature needs
 * to be specified at least filter and sub filter for signing; otherwise, this signature cannot be signed.
 *
 * @see @link interform::Field @endlink
 * @see @link interform::Form @endlink
 * @see @link PDFDoc @endlink
 * @see @link PDFPage @endlink
 */
class Signature FS_FINAL : public interform::Field {
 public:
  /**
   * @brief Enumeration for signature states.
   *
   * @details Values of this enumeration can be used alone or in combination.<br>
   *          These values can be divided into four parts:
   *          <ul>
   *          <li>Values starting from @link Signature::e_StateUnknown @endlink (with same prefix name) represent signed state,
   *              before signature is verified successfully.</li>
   *          <li>Values starting from @link Signature::e_StateVerifyValid @endlink (with same prefix name) represent
   *              the verified state of a signature.</li>
   *          <li>Values starting from @link Signature::e_StateVerifyIssueValid @endlink (with same prefix name) represent
   *              the verified state of issue for a signature, with more details.</li>
   *          <li>Values starting from @link Signature::e_StateVerifyTimestampNone @endlink (with same prefix name) represent
   *              the verified state for time stamp, with more details.</li>
   *          </ul>
   */
  typedef enum _States {
    /** @brief Unknown signature. */
    e_StateUnknown = 0x00000000,
    /** @brief Signature does not have any data for signing. */
    e_StateNoSignData = 0x00000200,
    /** @brief Unsigned signature.*/
    e_StateUnsigned = 0x00000001,
    /** @brief Signed signature. */
    e_StateSigned = 0x00000002,
    /** @brief Verification state of a signature is valid. */
    e_StateVerifyValid = 0x00000004,
    /** @brief Verification state of a signature is invalid. */
    e_StateVerifyInvalid = 0x00000008,
    /** @brief Signature data is destroyed (that means the signature data cannot be parsed properly). */
    e_StateVerifyErrorData = 0x00000010,
    /** @brief Unsupported signature. */
    e_StateVerifyNoSupportWay = 0x00000020,
    /** @brief Non expected byte range. */
    e_StateVerifyErrorByteRange = 0x00000040,
    /** @brief The document has been changed within the scope of the signature. */
    e_StateVerifyChange = 0x00000080,
    /** @brief Signature cannot be trusted (containing aggression). */
    e_StateVerifyIncredible = 0x00000100,
    /** @brief The document has not been changed within the scope of the signature. */
    e_StateVerifyNoChange = 0x00000400,
    /** @brief Verification state of the issuer is valid. */
    e_StateVerifyIssueValid = 0x00001000,
    /** @brief Verification state of the issuer is unknown. */
    e_StateVerifyIssueUnknown = 0x00002000,
    /** @brief Certificate for verifying issuer is revoked. */
    e_StateVerifyIssueRevoke = 0x00004000,
    /** @brief Certificate for verifying issuer is expired. */
    e_StateVerifyIssueExpire = 0x00008000,
    /** @brief Not check the issuer. */
    e_StateVerifyIssueUncheck = 0x00010000,
    /** @brief The verified issue is current issuer. */
    e_StateVerifyIssueCurrent = 0x00020000,
    /** @brief No timestamp or not check timestamp. */
    e_StateVerifyTimestampNone = 0x00040000,
    /** @brief The signature is a timestamp signature. */
    e_StateVerifyTimestampDoc = 0x00080000,
    /** @brief Verification state of the timestamp is valid. */
    e_StateVerifyTimestampValid = 0x00100000,
    /** @brief Verification state of the timestamp is invalid. */
    e_StateVerifyTimestampInvalid = 0x00200000,
    /** @brief Verification state of the timestamp is expired. */
    e_StateVerifyTimestampExpire = 0x00400000,
    /** @brief Verification state of the timestamp issuer is unknown. */
    e_StateVerifyTimestampIssueUnknown = 0x00800000,
    /** @brief Verification state of the timestamp issuer is valid */
    e_StateVerifyTimestampIssueValid = 0x01000000,
    /** @brief Verification state of the timestamp time is valid, since the times is before the expiration date. */
    e_StateVerifyTimestampTimeBefore = 0x02000000
  } States;
  
  /**
   * @brief Enumeration for signature appearance flags.
   *
   * @details Values of this enumeration can be used alone or in combination.
   */
  typedef enum _APFlags {
    /** @brief If set, show Foxit flag on signature appearance. */
    e_APFlagFoxitFlag = 0x0001,
    /** @brief If set, show label on signature appearance. */
    e_APFlagLabel = 0x0002,
    /** @brief If set, show reason on signature appearance. */
    e_APFlagReason = 0x0004,
    /** @brief If set, show signing time on signature appearance. */
    e_APFlagSigningTime = 0x0008,
    /** @brief If set, show distinguish name on signature appearance. */
    e_APFlagDN = 0x0010,
    /** @brief If set, show location on signature appearance. */
    e_APFlagLocation = 0x0020,
    /** @brief If set, show signer on signature appearance. */
    e_APFlagSigner = 0x0040,
    /** @brief If set, show bitmap on signature appearance. */
    e_APFlagBitmap = 0x0080,
    /** @brief If set, show text content on signature appearance. */
    e_APFlagText = 0x0100
  } APFlags;
  
  /**
   * @brief Enumeration for signature key name.
   *
   * @details Values of this enumeration should be used alone.
   */
  typedef enum _KeyName {
    /** @brief Signature key name: signer. */
    e_KeyNameSigner = 0,
    /** @brief Signature key name: location. */
    e_KeyNameLocation = 1,
    /** @brief Signature key name: reason. */
    e_KeyNameReason = 2,
    /** @brief Signature key name: contact information. */
    e_KeyNameContactInfo = 3,
    /** @brief Signature key name: distinguish name. */
    e_KeyNameDN = 4,
    /** @brief Signature key name: text content. */
    e_KeyNameText = 5
  } KeyName;
  
  /**
   * @brief Enumeration for signature digest algorithm.
   *
   * @details Values of this enumeration should be used alone.
   */
  typedef enum _DigestAlgorithm {
    /** @brief Signature digest algorithm: sha1 algorithm. */
    e_DigestSHA1 = 0,
    /** @brief Signature digest algorithm: sha256 algorithm. */
    e_DigestSHA256 = 1,
    /** @brief Signature digest algorithm: sha384 algorithm. */
    e_DigestSHA384 = 2,
    /** @brief Signature digest algorithm: sha512 algorithm. */
    e_DigestSHA512 = 3
  } DigestAlgorithm;
  

  /**
   * @brief Constructor, with parent class object.
   *
   * @param[in] field  Parent class object.
   */
  explicit Signature(const interform::Field& field);

  /**
   * @brief Constructor, from signature field dictionary.
   *
   * @param[in] document        A valid PDF document.
   * @param[in] sig_field_dict  A PDF dictionary which represents a signature field.
   *                            It should belong to the PDF document specified by parameter <i>document</i>;
   */
  Signature(const foxit::pdf::PDFDoc& document, foxit::pdf::objects::PDFDictionary* sig_field_dict);
  // User is strongly recommended NOT to use this method; otherwise unknown situation may occur.
  explicit Signature(FS_HANDLE handle = NULL);

  /** @brief Destructor. */
  ~Signature() {}

  /**
   * @brief Check whether current object is empty or not.
   *
   * @details When the current object is empty, that means current object is useless.
   *
   * @return <b>true</b> means current object is empty, while <b>false</b> means not.
   */
  bool IsEmpty() const;

  /**
   * @brief Check whether current signature is signed or not.
   *
   * @return <b>true</b> means current signature is signed, and <b>false</b> means not.
   */
  bool IsSigned();

  /**
   * @brief Start signing current signature if current signature is unsigned.
   *
   * @details This function is used for an unsigned signature. Filter and sub filter keys of current signature specify
   *          the name of signature callback object which will be used to sign current signature. So before signing,
   *          please ensure current signature has valid filter and sub filter string values. If the filter and sub filter
   *          are one of following strings that means to use the default signature callback in Foxit PDF SDK:
   *          <ul>
   *          <li>filter: "Adobe.PPKLite" and sub filter: "adbe.pkcs7.detached"</li>
   *          <li>filter: "Adobe.PPKLite" and sub filter: "adbe.pkcs7.sha1"</li>
   *          </ul>
   *          For other filter and sub filter, please ensure a customized signature callback with them has been
   *          registered to Foxit PDF SDK by function @link common::Library::RegisterSignatureCallback @endlink.<br>
   *          It may take a long time to sign a signature, so Foxit PDF SDK uses a progressive process to do this.
   *
   * @param[in] cert_path          A full path of a PFX certificate file (including file name and extension),
   *                               which will be used for signing. This can be an empty string if not necessary
   *                               in custom signature callback. When this file path is not empty, it should
   *                               be a valid path.
   * @param[in] cert_password      A password string used to open the cert file. If this is an empty string,
   *                               that means no password is required.
   * @param[in] digest_algorithm   The algorithm of message digest for signed data. Please refer to values
   *                               starting from @link Signature::e_DigestSHA1 @endlink and this should be one of these values.
   * @param[in] save_path          A full PDF file path for saving the signing result. The signed document would be
   *                               saved to another PDF file.
   * @param[in] client_data        A user-defined object, which will be passed to call back functions in
   *                               @link SignatureCallback @endlink. This is useless if the default callback object
   *                               will be used to sign current signature. Default value: <b>NULL</b>.
   * @param[in] pause              Pause object which decides if the signing process needs to be paused. This can be
   *                               <b>NULL</b> which means not to pause during the signing process. If this is
   *                               not <b>NULL</b>, it should be a valid pause object implemented by user.
   *                               Default value: <b>NULL</b>.
   *
   * @return A progressive object. Please check the rate of current progress by function
   *         @link common::Progressive::GetRateOfProgress @endlink. If the rate is not 100 yet, call function
   *         @link common::Progressive::Continue @endlink to coninue the progress until the progress is finished.
   *
   * @note This function does not support to save signed PDF document directly to the PDF file which is used to
   *       construct the related PDF document of current signature. In order to do so, user is recommended to
   *       do as following steps:<br>
   *       Assume that the related PDF document object is constructed from a PDF file named "org.pdf".
   *       <ol>
   *       <li>Use current function to save the signed result to an temporary file. Here, this temporary file is
   *           named as "temp.tmp".</li>
   *       <li>Ensure that the related PDF document object has destructed -- which is equal to "close document".</li>
   *       <li>Remove "org.pdf" and rename "temp.tmp" to "org.pdf".</li>
   *       </ol>
   *       Then user can open the signed PDF document to do other operation.
   */
  common::Progressive StartSign(const wchar_t* cert_path, const WString& cert_password,
                                DigestAlgorithm digest_algorithm, const char* save_path,
                                const void* client_data = NULL, common::PauseCallback* pause = NULL);

  /**
   * @brief Start signing current signature if current signature is unsigned.
   *
   * @details This function is used for an unsigned signature. Filter and sub filter keys of current signature specify
   *          the name of signature callback object which will be used to sign current signature. So before signing,
   *          please ensure current signature has valid filter and sub filter string values. If the filter and sub filter
   *          are one of following strings that means to use the default signature callback in Foxit PDF SDK:
   *          <ul>
   *          <li>filter: "Adobe.PPKLite" and sub filter: "adbe.pkcs7.detached"</li>
   *          <li>filter: "Adobe.PPKLite" and sub filter: "adbe.pkcs7.sha1"</li>
   *          </ul>
   *          For other filter and sub filter, please ensure a customized signature callback with them has been
   *          registered to Foxit PDF SDK by function @link common::Library::RegisterSignatureCallback @endlink.<br>:
   *          It may take a long time to sign a signature, so Foxit PDF SDK uses a progressive process to do this.
   *
   * @param[in] cert_path          A full path of a PFX certificate file (including file name and extension),
   *                               which will be used for signing. This can be an empty string if not necessary
   *                               in custom signature callback. When this file path is not empty, it should
   *                               be a valid path.
   * @param[in] cert_password      A password string used to open the cert file. If this is an empty string,
   *                               that means no password is required.
   * @param[in] digest_algorithm   The algorithm of message digest for signed data. Please refer to values
   *                               starting from @link Signature::e_DigestSHA1 @endlink and this should be one of these values.
   * @param[in] save_path          A full PDF file path for saving the signing result. The signed document would be
   *                               saved to another PDF file.
   * @param[in] client_data        A user-defined object, which will be passed to call back functions in
   *                               @link SignatureCallback @endlink. This is useless if the default callback object
   *                               will be used to sign current signature. Default value: <b>NULL</b>.
   * @param[in] pause              Pause object which decides if the signing process needs to be paused. This can be
   *                               <b>NULL</b> which means not to pause during the signing process. If this is
   *                               not <b>NULL</b>, it should be a valid pause object implemented by user.
   *                               Default value: <b>NULL</b>.
   *
   * @return A progressive object. Please check the rate of current progress by function
   *         @link common::Progressive::GetRateOfProgress @endlink. If the rate is not 100 yet, call function
   *         @link common::Progressive::Continue @endlink to coninue the progress until the progress is finished.
   *
   * @note This function does not support to save signed PDF document directly to the PDF file which is used to
   *       construct the related PDF document of current signature. In order to do so, user is recommended to
   *       do as following steps:<br>
   *       Assume that the related PDF document object is constructed from a PDF file named "org.pdf".
   *       <ol>
   *       <li>Use current function to save the signed result to an temporary file. Here, this temporary file is
   *           named as "temp.tmp".</li>
   *       <li>Ensure that the related PDF document object has destructed -- which is equal to "close document".</li>
   *       <li>Remove "org.pdf" and rename "temp.tmp" to "org.pdf".</li>
   *       </ol>
   *       Then user can open the signed PDF document to do other operation.
   */
  common::Progressive StartSign(const wchar_t* cert_path, const WString& cert_password,
                                DigestAlgorithm digest_algorithm, const wchar_t* save_path,
                                const void* client_data = NULL, common::PauseCallback* pause = NULL);

  /**
   * @brief Start verifying current signature if current signature is signed.
   *
   * @details This function is used for a signed signature. Filter and sub filter keys of current signature specify
   *          the name of signature callback object which will be used to verify current signature. So before verifying,
   *          please ensure: the necessary signature callback object has been registered by function
   *          @link common::Library::RegisterSignatureCallback @endlink. If the filter and sub filter of current signed signature
   *          are one of following strings that means to use the default signature callback in Foxit PDF SDK and
   *          user does not need to register one for them:
   *          <ul>
   *          <li>filter: "Adobe.PPKLite" and sub filter: "adbe.pkcs7.detached"</li>
   *          <li>filter: "Adobe.PPKLite" and sub filter: "adbe.pkcs7.sha1"</li>
   *          </ul>
   *          It may take a long time to verify a signature, so Foxit PDF SDK uses a progressive process to do this.
   *
   * @param[in] client_data  A user-defined object, which will be passed to call back functions in
   *                         @link SignatureCallback @endlink. This is useless if the default callback object will be
   *                         used to verify current signature. Default value: <b>NULL</b>.
   * @param[in] pause        Pause object which decides if the verifying process needs to be paused.
   *                         This can be <b>NULL</b> which means not to pause during the verifying process.
   *                         If this is not <b>NULL</b>, it should be a valid pause object implemented by user.
   *                         Default value: <b>NULL</b>.
   *
   * @return A progressive object. Please check the rate of current progress by function
   *         @link common::Progressive::GetRateOfProgress @endlink. If the rate is not 100 yet, call function
   *         @link common::Progressive::Continue @endlink to coninue the progress until the progress is finished.
   */
  common::Progressive StartVerify(const void* client_data = NULL, common::PauseCallback* pause = NULL);

  /**
   * @brief Get certificate information.
   *
   * @details This function is used for a signed signature.
   *          Currently, this function only supports for iOS and android platform.
   *
   * @param[in] key  Certificate key string. Currently it can be one of the following keys:<br>
   *                 <ul>
   *                 <li>"SerialNumber"</li>
   *                 <li>"Issuer"</li>
   *                 <li>"Subject"</li>
   *                 <li>"ValidPeriodFrom"</li>
   *                 <li>"ValidPeriodTo"</li>
   *                 </ul>
   *
   * @return Certificate information string.
   *
   * @note For "ValidPeriodFrom" or "ValidPeriodTo" key, timezone value will not be computed in.
   */
  String GetCertificateInfo(const char* key);

  /**
   * @brief Get the byte ranges data, including 4 elements.
   *
   * @details This function is used for a signed signature to retrieve its byte range for digest calculation.
   *          The array of byte ranges contains 4 elements. These 4 elements are always in pairs of
   *          integers (starting byte offset, length in bytes), describing the exact byte range
   *          for the digest calculation.<br>
   *          Please refer to <PDF Reference 1.7> Section 8.7 Digital Signatures for more details.
   *
   * @param[out] out_byte_range_array  Output parameter that receives data of byte ranges.
   *                                   Please ensure this array can contain 4 elements.
   *
   * @return <b>true</b> means success, while <b>false</b> means failure.
   */
  bool GetByteRangeArray(uint32 out_byte_range_array[4]);

  /**
   * @brief Get current state.
   *
   * @details <ul>
   *          <li>Before verifying a signature, this function is to get the state about if current signature is signed
   *              or if current signature is lack of data for signing.</li>
   *          <li>After verifying a signature, this function is to get the verified state -- which indicates that
   *              the verified signature is signed; if the verified signature is still unsigned, this function will
   *              get the unsigned state instead.</li>
   *          </ul>
   *
   * @return The value of signature state. Please refer to values starting from @link Signature::e_StateUnknown @endlink and
   *         this would be one or combination of them.<br>
   *         Specially, if the returned state is @link Signature::e_StateNoSignData @endlink,
   *         that means current signature has no data for signing, and please at least call function
   *         @link Signature::SetKeyValue @endlink to set necessary filter and sub filter to current signature.
   *         Use can also call following functions to set other data for signing and signed appearance before calling
   *         function @link Signature::StartSign @endlink:<br>
   *         <ul>
   *         <li>For using default Foxit appearance template, related information can be set by
   *             @link Signature::SetAppearanceFlags @endlink, @link Signature::SetSignTime @endlink,
   *             @link Signature::SetKeyValue @endlink, @link Signature::SetBitmap @endlink, @link Signature::SetImage @endlink.</li>
   *         <li>For using customized appearance, please set appearance stream content by function
   *             @link Signature::SetAppearanceContent @endlink.</li>
   *         </ul>
   */
  uint32 GetState();

  /**
  * @brief Clear the data and appearance if current signature is singed and verified valid.
  *
  * @details This function is used for a signed and valid signature.
  *
  * @return <b>true</b> means clear the data and appearance successfully, while <b>false</b> means no need to
  *         clear data and appearance or any error.
  */
  bool ClearSignedData();

  /**
  * @brief Get the PDF document, which current signature belongs to.
  *
  * @return A PDF document object.
  */
  PDFDoc GetDocument();

  /**
  * @brief Get signature appearance flags.
  *
  * @details Signature appearance flags indicate which information will be shown.
  *          Currently, this is only useful after @link Signature::SetAppearanceFlags @endlink is set successfully.
  *          For a signature gotten from PDF document, the return value of this function would be useless.
  *
  * @return Signature appearance flags. Please refer to values starting from @link Signature::e_APFlagFoxitFlag @endlink and
  *         this would be one or a combination of these values.
  */
  uint32 GetAppearanceFlags();

  /**
  * @brief Set signature appearance flags.
  *
  * @details This function is recommended to be used before calling function @link Signature::StartSign @endlink for
  *          an unsigned signature. <br>
  *          Signature appearance flags indicate which information will be shown in the signed appearance.
  *          If customized appearance content has been set by function @link Signature::SetAppearanceContent @endlink,
  *          appearance flags will be ignored.
  *
  * @param[in] appearance_flags  Signature appearance flags. Please refer to values starting from
  *                              @link Signature::e_APFlagFoxitFlag @endlink and this should be
  *                              one or a combination of these values.
  *
  * @return None.
  */
  void SetAppearanceFlags(uint32 appearance_flags);

  /**
  * @brief Set customized appearance content (as low level drawing operation commands)
  *        for signed signature appearance.
  *
  * @details This function can only be used for an unsigned signature before calling function
  *          @link Signature::StartSign @endlink for the unsigned signature.<br>
  *          Once customized appearance content is set, it will be used as the signed appearance. In this case,
  *          appearance flags set by @link Signature::SetAppearanceFlags @endlink will be ignored and
  *          related informations will not be used in appearance -- these information are set by:
  *          @link Signature::SetSignTime @endlink, @link Signature::SetKeyValue @endlink (except filter and sub-filter),
  *          @link Signature::SetBitmap @endlink, @link Signature::SetImage @endlink.
  *
  * @param[in] appearance_content  Customized appearance content. This should be sequence of drawing operation commands
  *                                to be used for the appearance, for example "10 10 m 20 10 l S". Please refer to
  *                                <PDF Reference 1.7> P196 for more details.
  *
  * @return None.
  */
  void SetAppearanceContent(const String& appearance_content);

  /**
  * @brief Get time of signing.
  *
  * @return A date and time object that receives the signing time.
  *         If no signing time is found, this function will return a date and time object with all values 0.
  */
  DateTime GetSignTime();

  /**
  * @brief Set time of signing.
  *
  * @details This function is recommended to be used before calling function @link Signature::StartSign @endlink for
  *          an unsigned signature.
  *
  * @param[in] sign_time  The signing time.
  *
  * @return None.
  */
  void SetSignTime(const DateTime& sign_time);

  /**
   * @brief Get filter.
   *
   * @details Filter and sub filter are used to specify which registered signature callback object will be used to
   *          sign/verify current signature.
   *
   * @return Filter string.
   */
  String GetFilter();
  /**
   * @brief Set filter.
   *
   * @details Filter and sub filter are used to specify which registered signature callback object will
   *          be used to sign/verify current signature. User could should set filter and sub filter to
   *          use default signature callback in Foxit PDF SDK, or use other filter and sub fitler but please
   *          ensure that a signature callback object with non-default filter and sub filter should has been
   *          registered in Foxit PDF SDK before signing or verifying current signature.
   *          Filter and sub filter for default signature callback are:
   *          <ul>
   *          <li>filter: "Adobe.PPKLite" and sub filter: "adbe.pkcs7.detached"</li>
   *          <li>filter: "Adobe.PPKLite" and sub filter: "adbe.pkcs7.sha1"</li>
   *          </ul>
   *
   * @param[in] filter  String for filter. It cannot be an empty string.
   *
   * @return None.
   */
  void SetFilter(const char* filter);

  /**
   * @brief Get sub filter.
   *
   * @details Filter and sub filter are used to specify which registered signature callback object will be used to
   *          sign/verify current signature.
   *
   * @return Sub filter string.
   */
  String GetSubFilter();
  /**
   * @brief Set sub filter.
   *
   * @details Filter and sub filter are used to specify which registered signature callback object will
   *          be used to sign/verify current signature. User could should set filter and sub filter to
   *          use default signature callback in Foxit PDF SDK, or use other filter and sub fitler but please
   *          ensure that a signature callback object with non-default filter and sub filter should has been
   *          registered in Foxit PDF SDK before signing or verifying current signature.
   *          Filter and sub filter for default signature callback are:
   *          <ul>
   *          <li>filter: "Adobe.PPKLite" and sub filter: "adbe.pkcs7.detached"</li>
   *          <li>filter: "Adobe.PPKLite" and sub filter: "adbe.pkcs7.sha1"</li>
   *          </ul>
   *
   * @param[in] sub_filter  String for sub filter.
   *
   * @return None.
   */
  void SetSubFilter(const char* sub_filter);

  /**
   * @brief Get the string value for specified key name.
   *
   * @details This function is used to get string value of some key in signature dictionary, such as "Reason",
   *          "Location" and so on. Specially,
   *
   * @param[in] key  Key name. Please refer to values starting from @link Signature::e_KeyNameSigner @endlink and
   *                 this should be one of these values.
   *
   * @return The string value.
   */
  WString GetKeyValue(KeyName key);

  /**
   * @brief Set the string value for specified key name.
   *
   * @details This function is recommended to be used before calling function @link Signature::StartSign @endlink for
   *          an unsigned signature.<br>
   *          This function is used to set string value of some key in signature dictionary, such as signer, reason,
   *          location and so on.
   *
   * @param[in] key    Key name. Please refer to values starting from @link Signature::e_KeyNameSigner @endlink and
   *                   this should be one of these values.
   * @param[in] value  New string value.
   *
   * @return None.
   */
  void SetKeyValue(KeyName key, const wchar_t* value);

  /**
   * @brief Get a bitmap which is used for the signature appearance.
   *
   * @return The bitmap used in appearance. If the return value of function @link common::Bitmap::IsEmpty @endlink for
   *         the returned bitmap object is <b>true</b>, that means no bitmap is used in appearance or there is any error.
   */
  common::Bitmap GetBitmap();

  /**
   * @brief Set a bitmap for the signature appearance.
   *
   * @details This function is recommended to be used before calling function @link Signature::StartSign @endlink for
   *          an unsigned signature. If customized appearance content has been set by function
   *          @link Signature::SetAppearanceContent @endlink, the bitmap will not be used in signed appearance.
   *
   * @param[in] bitmap  A bitmap to be set to the appearance. It should be valid.
   *
   * @return None.
   */
  void SetBitmap(const common::Bitmap& bitmap);

  /**
   * @brief Set an image for the signature appearance, with a specified frame index.
   *
   * @details This function is recommended to be used before calling function @link Signature::StartSign @endlink for
   *          an unsigned signature. If customized appearance content has been set by function
   *          @link Signature::SetAppearanceContent @endlink, the image will not be used in signed appearance.<br>
   *          Input image may contain multiple frames, and only one frame of the image can be set to current signature.
   *
   * @param[in] image        An image. One of its frames will be set to current signature. This image contains at least
   *                         one frame and the image type should not be @link common::Image::e_Unknown @endlink.
   * @param[in] frame_index  Frame index. Valid range: from 0 to (<i>count</i>-1).
   *                         <i>count</i> is returned by function @link common::Image::GetFrameCount @endlink.
   *
   * @return None.
   */
  void SetImage(const common::Image& image, int frame_index);

  /**
   * @brief Set an image for the signature appearance, with a specified frame index.
   *
   * @details This function is recommended to be used before calling function @link Signature::StartSign @endlink for
   *          an unsigned signature. If customized appearance content has been set by function
   *          @link Signature::SetAppearanceContent @endlink, the image will not be used in signed appearance.<br>
   *          Input image may contain multiple frames, and only one frame of the image can be set to current signature.
   *
   * @param[in] file_path    A full path of an existing image file. It should not be an empty string.
   * @param[in] frame_index  Frame index. Valid range: from 0 to (<i>count</i>-1).
   *                         <i>count</i> is returned by function @link common::Image::GetFrameCount @endlink
   *                         of input image file.
   *
   * @return None.
   */
  void SetImage(const char* file_path, int frame_index);

  /**
   * @brief Set an image for the signature appearance, with a specified frame index.
   *
   * @details This function is recommended to be used before calling function @link Signature::StartSign @endlink for
   *          an unsigned signature. If customized appearance content has been set by function
   *          @link Signature::SetAppearanceContent @endlink, the image will not be used in signed appearance.<br>
   *          Input image may contain multiple frames, and only one frame of the image can be set to current signature.
   *
   * @param[in] file_path    A full path of an existing image file. It should not be an empty string.
   * @param[in] frame_index  Frame index. Valid range: from 0 to (<i>count</i>-1).
   *                         <i>count</i> is returned by function @link common::Image::GetFrameCount @endlink
   *                         of input image file.
   *
   * @return None.
   */
  void SetImage(const wchar_t* file_path, int frame_index);

  /**
   * @brief Get signature dictionary.
   *
   * @details Signature dictionary is a part of signature field dictionary.
   *
   * @return The signature dictionary.
   */
  objects::PDFDictionary* GetSignatureDict() const;

  /**
  * @brief Set the default length of signature contents which represents signature value (known as signed data).
  *
  * @details This function can only be used before function @link Signature::StartSign @endlink
  *          for an unsigned signature.<br>
  *          Signature contents represent the signature value (known as signed data).
  *          When initializing the signature field, the signature contents will be initialized with
  *          the default length 7942. When using custom signature callback to sign and verify an unsigned signature,
  *          user can call this function to change the default length of signature contents. In this case,
  *          user should also ensure that the length of returned signed data in callback function
  *          @link SignatureCallback::Sign @endlink should not be larger than the default length set by this function.
  *
  * @param[in] default_length  The default length of the signature value (known as signed data), in bytes.
  *                            It should not be less than 4098.
  *
  * @return None.
  */
  void SetDefaultContentsLength(uint32 default_length);

  /**
   * @brief Get the count of certificates in current signature's certificate chain.
   *
   * @details This function is recommended to be used before verifying a signature.
   *          When sub filter of current signature is "adbe.x509.rsa_sha1", application needs to call this function to
   *          get the count of certificates in certificate chain and then call function @link Signature::GetCert @endlink
   *          to get verified public certificate before verifying signature.
   *
   * @return The count of certificates.
   */
  int32 GetCertCount() const;

  /**
   * @brief Get a certificate from current signature's certificate chain by index.
   *
   * @details This function is recommended to be used before verifying a signature.
   *          When sub filter of current signature is "adbe.x509.rsa_sha1", application needs to call this function to
   *          get verified public certificate before verifying signature.
   *          The first certificate in certificate chain is the signing certificate, and it can be used to
   *          verify the signature.
   *
   * @param[in] index  The index of certificate to be gotten. Valid range: from 0 to (<i>count</i>-1).
   *                   <i>count</i> is returned by function @link Signature::GetCertCount @endlink.
   *
   * @return A certificate.
   */
  WString GetCert(int32 index) const;

  /**
   * @brief Set a certificate chain.
   *
   * @details This function is recommended to be used before signing an unsigned signature.
   *          When the signature sub filter is "adbe.x509.rsa_sha1", users need to call this function to
   *          set the cert chain to the signature. For other sub filter, this function will return directly without
   *          doing anything.
   *
   * @param[in] cert_chain  A string array that represents the certificate chain.
   *
   * @return None.
   */
  void SetCertChain(const WStringArray& cert_chain);
};

/**
 * This class represent a callback object to sign or verify signature.
 * All the pure virtual functions in this class are used as callback functions and should be implemented by user,
 * in order to sign or verify signature in a customized way.<br>
 * User can set register customized signature callback object to Foxit PDF SDK by function
 * @link common::Library::RegisterSignatureCallback @endlink with filter and sub filter as the callback name.
 *
 * @see @link common::Library @endlink
 */
class SignatureCallback {
 public:
  /**
   * @brief A callback function used to release current callback object itself.
   *
   * @return None.
   */
  virtual void Release() = 0;

  /**
   * @brief A callback function used to start a custom digest calculation, mainly to initialize the calculation process.
   *
   * @param[in] file              When signing a signature, this represents the source file object to be signed.<br>
   *                              When verifying a signature field, this represents the source file object
   *                              to be verified.
   * @param[in] byte_range_array  A byte range array of the signed document(not the source document).<br>
   *                              Elements in this array always be in pairs of offset-size values in order of:
   *                              offset,size,offset,size...
   * @param[in] size_of_array     Count of elements in the byte range array <i>byte_range_array</i>.
   * @param[in] signature         A signature object.
   * @param[in] client_data       A user-defined object, which is passed by user from function
   *                              @link Signature::StartSign @endlink or @link Signature::StartVerify @endlink.
   *
   * @return <b>true</b> means the calculation process is successfully started and then
   *         @link SignatureCallback::ContinueCalcDigest @endlink will be triggered to continue the calculation process.
   *         <b>false</b> means fail to start a calculation process.
   */
  virtual bool StartCalcDigest(const common::file::ReaderCallback* file, const uint32* byte_range_array,
                               uint32 size_of_array, const Signature& signature, const void* client_data) = 0;

  /**
   * @brief A callback function used to continue the custom digest calculation process.
   *
   * @param[in] pause        Pause object which decides if the calculation process needs to be paused.
   *                         If this is <b>NULL</b>, it means not to pause during the calculation process.
   *                         If this is not <b>NULL</b>, please check the the return value of
   *                         function NeedToPauseNow() of this input parameter and decide when to pause
   *                         the calculation process.
   * @param[in] client_data  A user-defined object, which is passed by user from function
   *                         @link Signature::StartSign @endlink or @link Signature::StartVerify @endlink.
   *
   * @return @link common::Progressive::e_Finished @endlink means the rendering is finished
   *         successfully.<br>
   *         @link common::Progressive::e_ToBeContinued @endlink means the calculation
   *         process is not finished yet and callback function @link SignatureCallback::ContinueCalcDigest @endlink
   *         would be triggered again to continue the process.<br>
   *         @link common::Progressive::e_Error @endlink means any error occurs.
   */
  virtual common::Progressive::State ContinueCalcDigest(const void* client_data,
                                                          const common::PauseCallback* pause) = 0;

  /**
   * @brief A callback function used to get the digest after the calculation process is finished.
   *
   * @details This callback function will be triggered when the calculation process is finished successfully.
   *
   * @param[in] client_data  A user-defined object, which is passed by user from function
   *                         @link Signature::StartSign @endlink or @link Signature::StartVerify @endlink.
   *
   * @return The digest string.
   *         If there is any error, please return an empty string.
   */
  virtual String GetDigest(const void* client_data) = 0;

  /**
   * @brief A callback function used to do signing and return the signed data.
   *
   * @details Host application provides this callback function for custom sign mechanism.
   *          This callback function would be triggered during the signature signing process.
   *
   * @param[in] digest            The digest data buffer, for source file buffer to be signed.
   * @param[in] digest_length     The size of digest data buffer.
   * @param[in] cert_path         The certificate file path, which will be used for signing.
   * @param[in] cert_password     The password string, used to open the cert file. If this is an empty string,
   *                              that means no password is required.
   * @param[in] digest_algorithm  The algorithm of message digest for signed data. Please refer to values
   *                              starting from @link Signature::e_DigestSHA1 @endlink and this would be one of these values.
   * @param[in] client_data       A user-defined object, which is passed by user from function
   *                              @link Signature::StartSign @endlink or @link Signature::StartVerify @endlink.
   *
   * @return Signed data. If there is any error, please return an empty string.<br>
   *         The length of returned signed data should be within the default length of signature contents
   *         which is 7942 by default or specified by fucntion @link Signature::SetDefaultContentsLength @endlink
   *         before signing the unsigned signature.
   */
  virtual String Sign(const void* digest, uint32 digest_length, const wchar_t* cert_path,
                      const WString& cert_password, Signature::DigestAlgorithm digest_algorithm,
                      void* client_data) = 0;

  /**
   * @brief A callback function used to do verifying and return the verify state.
   *
   * @details Host application provides this callback function for custom sign mechanism.
   *          This callback function would be triggered during the signature verifying process.
   *
   * @param[in] digest           The digest data buffer, for source file buffer to be verified.
   * @param[in] digest_length    The size of digest data buffer.
   * @param[in] signed_data      The signed data buffer.
   * @param[in] signed_data_len  The size of signed data buffer.
   * @param[in] client_data      A user-defined object, which is passed by user from function
   *                             @link Signature::StartSign @endlink or @link Signature::StartVerify @endlink.
   *
   * @return Verify state. Please refer to values starting from @link Signature::e_StateVerifyValid @endlink and
   *         this should be one or a combination of them.
   */
  virtual uint32 VerifySigState(const void* digest, uint32 digest_length,
                                const void* signed_data, uint32 signed_data_len,
                                void* client_data) = 0;

  /**
   * @brief Check whether to return all contents with padding zero or not, when callback function
   *        @link SignatureCallback::VerifySigState @endlink is called.
   *
   * @return <b>true</b> means to return all data in contents steam (including padding zero),
   *         while <b>false</b> means to return contents data without padding zero in stream end.
   */
  virtual bool IsNeedPadData() = 0;

 protected:
  ~SignatureCallback() {}
};
}  // namespace pdf
}  // namespace foxit
#endif  // FS_SIGNATURE_H_

